package IkiWiki::Plugin::hl;

use warnings;
use strict;

use IkiWiki 2.00;
use File::Temp;

# Variables 
our $VERSION = '0.05';

my $exec_cmd = q(/usr/bin/highlight --quiet --force --xhtml --fragment --linenumbers);
my %def_parameters = ( 
    syntax      =>      undef,
    source      =>      undef,
    file        =>      undef,
    linenumbers =>      1,
);

# Code
sub import {
    hook ( type => "preprocess", id => 'hl', call => \&preprocess );
}

=pod

In a ikiwiki source page we found 

[[!hl syntax="ini" source="""

# This is my first config file 
root = /tmp

"""]] 

=cut

sub preprocess {
    my  %params =   ( %def_parameters, @_ );
    my  $html   = '';

    # clean the parameters
    $params{syntax} =~ s/[^\w]+//g if $params{syntax};
    
    # get the source 
    my  $filename   =   $params{file};
    my  $fh         =   undef;

    if (not defined $filename) {
        $fh = File::Temp->new();
        
        if ($fh) {
            $fh->print( $params{source} );
            $fh->close();

            $filename = $fh->filename();
        }
        else {
            error("could not open a temporary file");
        }
    }
    else {
        # take the real path to the page 
        $filename = srcfile($params{file});
    }

    # run the highlight program 
    $html = _run_highlight( $filename, lc $params{syntax} );

    # clean the html output
    $html = sprintf("<pre>\n%s</pre>\n", $html);

    # and return 
    return $html;
}

sub _run_highlight {
    my  ( $input_file, $syntax ) = @_;

    my  $syntax_type = $syntax 
                        ? sprintf('--syntax=%s', $syntax)
                        : '';
    my  $cmd = sprintf('%s %s < %s ', $exec_cmd, $syntax_type, $input_file );
    my  $ret = '';

    if (open(HL, "$cmd |")) {
        while (<HL>) {
            $ret .= $_;
        }
        close HL;
    }
    else {
        die "could not execute $cmd - $!";
    }

    return $ret;
}

1; 
